'************************* AX2D.BAS *************************
'  U. S. Geological Survey Computer Program Module AX2D.BAS
'       device-independent routines for graphics system
' $INCLUDE: 'disclaim.inc'
'*************************************************************************
' $INCLUDE: 'jk2dplt.inc'
' $INCLUDE: 'jk2dint.inc'
' $INCLUDE: 'ax2d.inc'
' $INCLUDE: 'ax2dint.inc'

DIM xprm AS AxprmT
DIM yprm AS AxprmT
DIM SHARED axpg AS AxpgT
        DEF fnmin (a, b) = -b * (b < a) - a * (a < b) - a * (a = b)

END

SUB autoloc (page, spos, ax AS AxprmT)
'checks that axis location and length are acceptable
'if not, it tries to place axis position and length within page
        IF spos >= 0 AND spos < page THEN
          IF ax.axis <= 0 OR spos + ax.axis > page THEN
            ax.axis = page - spos
          END IF
        ELSE
          IF ax.axis > 0 AND ax.axis < page THEN
            spos = .5 * (page - ax.axis)
          ELSE
            ax.axis = page - 16 * ax.ht
            spos = 8 * ax.ht
            IF ax.axis <= 0 THEN
              ax.axis = .75 * page
              spos = .125 * page
            END IF
          END IF
        END IF
END SUB

FUNCTION autoplt% (xpage, ypage, xmin, xmax, ymin, ymax, errmsg$)
'automatic setting or plotting of both axes and scaling
'parms  xpage,ypage     plot page size in inches
'       xmin,xmax       horizontal axis limits
'       ymin,ymax       vertical axis limits
'       errmsg$         returned error message
'returns 0 if ok, -1 if not
'-------------------------------------------------------------------------
SHARED xprm AS AxprmT, yprm AS AxprmT
        xprm.which = "X"
        yprm.which = "Y"
'always check initialization
        IF NOT xprm.set THEN xprm.org = 0: xprm.scl = 1
        IF NOT yprm.set THEN yprm.org = 0: yprm.scl = 1
        errmsg$ = "": axpg.eps = .000001
'make sure axis types are acceptable
        IF xprm.typ <> 1 AND xprm.typ <> 2 AND xprm.typ <> 3 THEN xprm.typ = 1
        IF yprm.typ <> 1 AND yprm.typ <> 2 AND yprm.typ <> 3 THEN yprm.typ = 1
'save nodraw variables so regular ones can be used for precalculation
        xhld% = xprm.nodraw
        yhld% = yprm.nodraw
'all we have to do is insure rounded axis calculation
        xprm.stp = 0: xprm.xstep = 0: xprm.cycle = 0
        yprm.stp = 0: yprm.xstep = 0: yprm.cycle = 0
'precalculate for equal scaling case
        IF axpg.eqplt THEN
'if axis types are not the same, ignore equal scaling instruction
          IF xprm.typ = yprm.typ THEN
'preliminary scale calculations
            tmp% = axis2%(xpage, ypage, xmin, xmax, ymin, ymax, -1, errmsg$)
            SELECT CASE xprm.typ
              CASE 1            'GRAF
                IF xprm.scl >= yprm.scl THEN
                  xprm.xend = xprm.strt + xprm.axis * (yprm.xend - yprm.strt) / yprm.axis
                ELSE
                  yprm.xend = yprm.strt + yprm.axis * (xprm.xend - xprm.strt) / xprm.axis
                END IF
'                 xmin = xprm.strt
'                 ymin = yprm.strt
'                 xmax = xprm.xend
'                 ymax = yprm.xend
'                xmin = (21 * xprm.strt + xprm.xend) / 22 + .5 * xprm.stp
'                xmax = (xprm.strt + 21 * xprm.xend) / 22 - .5 * xprm.stp
'                ymin = (21 * yprm.strt + yprm.xend) / 22 + .5 * yprm.stp
'                ymax = (yprm.strt + 21 * yprm.xend) / 22 - .5 * yprm.stp
'                xprm.stp = 0: yprm.stp = 0
                 IF xprm.stp >= yprm.stp THEN
                   yprm.stp = xprm.stp
                 ELSE
                   xprm.stp = yprm.stp
                 END IF
              CASE 2            'GRAPH
                IF xprm.xstep >= yprm.xstep THEN
                  xprm.xstep = yprm.xstep
                ELSE
                  yprm.xstep = xprm.xstep
                END IF
              CASE 3            'LOG
                IF xprm.cycle >= yprm.cycle THEN
                  xprm.cycle = yprm.cycle
                ELSE
                  yprm.cycle = xprm.cycle
                END IF
              CASE ELSE
            END SELECT
          ELSE
            errmsg$ = errmsg$ + "/EQPLT"
          END IF
        END IF
        xprm.set = 0: yprm.set = 0
        xprm.nodraw = xhld%: yprm.nodraw = yhld%
        autoplt% = axis2%(xpage, ypage, xmin, xmax, ymin, ymax, 0, errmsg$)
END FUNCTION

FUNCTION axis1% (ax AS AxprmT, opos)
'draw single axis-returns 0 if ok, -1 if not
'does not try to fix errors in input parameters
'parameters
'for all types
'       typ,xpos,ypos,axis,which$,ht,side,dec)
'for type 1
'       strt,stp,xend
'for type 2
'       strt,xstep
'for type 3 (log)
'       strt,cycle
'******* permanent axis prms stored for x and y axes (example for x)
'       typ,org,axis,strt,stp,scale,stk
'-------------------------------------------------------------------------
'********* typ/scl routines
d4% = 4
        DIM dx(d4%), dy(d4%)
'*** common calculations, defaults
        eps = .000001
        tcktol = .05
        ier% = 0
        IF ax.axis <= 0 THEN
          ier% = -1
        ELSE
          logt% = 0
          IF ax.ht = 0 THEN ax.ht = .0125 * axpg.xpage
          IF ax.side = 0 THEN ax.side = 1
          IF UCASE$(ax.which) = "X" THEN
            ax.org = ax.xpos
          ELSE
            ax.org = ax.ypos
          END IF
          SELECT CASE ax.typ
'case type 1
            CASE 1
              IF (ax.xend - ax.strt) * ax.stp > 0 THEN
                ax.scl = ax.axis / (ax.xend - ax.strt)
              ELSE
                ier% = -1
              END IF
            CASE 2
'case type 2
              IF ax.xstep <> 0 THEN
                ax.stp = ax.xstep
                ax.scl = 1 / ax.xstep
              ELSE
                ier% = -1
              END IF
            CASE 3
 'case type 3
              logt% = -1
              IF ax.cycle <> 0 AND ax.strt > 0 THEN
                ax.scl = ax.cycle
                ax.stp = SGN(ax.cycle)
              ELSE
                ier% = -1
              END IF
            CASE ELSE
              ier% = -1
          END SELECT
        END IF
        IF NOT ier% THEN
          ax.set = -1
          smul = .5
          lab$ = LTRIM$(RTRIM$(ax.lab))
'set tick-drawing parms
          CALL settick(ax, opos, dx(), dy(), ang, xs, ys, xe, ye, lab$)
'main tick loop
          CALL tickloop(ax, dx(), dy(), ang, xs, ys, xe, ye, .5)
        END IF
'axis labeling
        IF NOT ax.grd AND NOT ax.nodraw THEN
          IF NOT ax.nolab AND lab$ <> "" THEN
            xp = xs + dx(4)
            yp = ys + dy(4)
            CALL Symbol(xp, yp, 2 * ax.ht, lab$, ang)
          END IF
        END IF
        axis1% = ier%
ERASE dx, dy
END FUNCTION

FUNCTION axis2% (xpage, ypage, xmin, xmax, ymin, ymax, calconly%, errmsg$)
'requires the prms for both axes - as saved later
'returns 0 if ok, -1 if not
'need xpage, ypage and max's and min's for default calculations & comparisons
'shared parms
'       xpos,ypos    :origin - if neg, is calculated
'       typ          :axis type - default to type 1
'       axis        :axis length, if 0 or neg, calculated
'       ht           :base annot ht, if <=0, set to .0125*xpage
'       dec          :decimals for annotation - calc for rounded axes
'  ------
'       strt         :axis definition -
'       xend         :stp/step/cycle=0 is key for increment calc
'       stp          :min<max is key for rounded limits if inc calc
'       xstep        :for graf case if strt=end, inc is calc
'       cycle        :for non-rounded limits, inc is dif or len/5
' --------
'       nolab        :can be specified to omit label
'       lab          :axis labels - default=blank
'       sd           :axis sides - 0,1=standard, -1=other,else both
'       stck         :sub ticks for annotation-default=0
' --------
'       title        :title - default is blank
'       frame        :can be set to 0 or -1
'       grd          :can be set if wanted
'-------------------------------------------------------------------------
SHARED xprm AS AxprmT, yprm AS AxprmT
        xnh% = xprm.nodraw
        ynh% = yprm.nodraw
        xprm.which = "X"
        yprm.which = "Y"
'check for position of axes and calculate if necessary
        IF calconly% THEN xprm.nodraw = -1: yprm.nodraw = -1
'calculate annotation heights for standard configuation if necessary
        xtmp% = 0
        IF xprm.ht <= 0 THEN
          xprm.ht = .0125 * xpage
          xtmp% = -1
        END IF
        ytmp% = 0
        IF yprm.ht <= 0 THEN
          yprm.ht = .0125 * ypage
          ytmp% = -1
        END IF

'xaxis
'check for autolocation
        CALL autoloc(xpage, xprm.xpos, xprm)
        yprm.xpos = xprm.xpos
'yaxis
'check for autolocation
        CALL autoloc(ypage, yprm.ypos, yprm)
        xprm.ypos = yprm.ypos
'positioned - now set axis limits in user units
'recheck automatic height to allow for all annotation
        IF xtmp% AND xprm.ypos < 8 * xprm.ht THEN xprm.ht = xprm.ypos / 8
        IF ytmp% AND yprm.xpos < 8 * yprm.ht THEN yprm.ht = yprm.xpos / 8
        IF axpg.ht <= 0 THEN axpg.ht = SQR(xprm.ht * yprm.ht)
'x axis
        a% = checkax%(xpage, xmin, xmax, xprm)
        tmp1% = axis1%(xprm, yprm.ypos)
'y axis
        a% = checkax%(ypage, ymin, ymax, yprm)
        tmp2% = axis1%(yprm, xprm.xpos)
'title
        IF xprm.set AND yprm.set THEN
          lab$ = LTRIM$(RTRIM$(axpg.title))
          IF lab$ <> "" THEN
            xp = xprm.org + .5 * xprm.axis - 1.25 * axpg.ht * LEN(lab$)
            yp = yprm.org + yprm.axis + 4 * axpg.ht
            CALL Symbol(xp, yp, 2.5 * axpg.ht, lab$, 0)
          END IF
        END IF
        xprm.nodraw = xnh%
        yprm.nodraw = ynh%
        axis2% = tmp1% OR tmp2%
END FUNCTION

FUNCTION checkax% (page, min, max, prm AS AxprmT)
'checks for consistency of axis parms - sets to defaults if trouble
'returns 0 if ok, -1 if not
        ier% = 0
        logt% = 0
        thru% = -1
        IF prm.typ <> 1 AND prm.typ <> 2 AND prm.typ <> 3 THEN prm.typ = 1
        DO
          SELECT CASE prm.typ
'type 1 axis
            CASE 1
              thru% = -1
              IF prm.strt = prm.xend OR prm.stp = 0 THEN
                IF min >= max THEN
'impossible situation - try expedient sol but flag error
                  ier% = -1
                  prm.strt = 0
                  prm.xend = page
                  prm.stp = page / 5
                  prm.dec = setdec%(prm.stp)
                ELSE
                  a% = roundax%(min, max, lb, inc, ub, dec%, logt%)
                  prm.strt = lb
                  prm.xend = ub
                  prm.stp = inc
                  prm.dec = dec%
                END IF
              ELSE
                IF (prm.xend - prm.strt) * prm.stp < 0 THEN
                  prm.stp = (prm.xend - prm.strt) / 5
                  prm.dec = setdec%(prm.stp)
                END IF
              END IF
            CASE 2
'type 2 axis
              IF prm.xstep = 0 THEN
                IF min < max THEN
                  a% = roundax%(min, max, lb, inc, ub, dec%, 0)
                  prm.strt = lb
                  prm.xstep = (ub - lb) / prm.axis
                  prm.dec = setdec%(prm.xstep)
                ELSEIF prm.strt <> prm.xend THEN
                    prm.xstep = (prm.xend - prm.strt) / prm.axis
                    prm.dec = setdec%(prm.xstep)
                ELSE
'impossible situation - expedient try
                  ier% = -1
                  prm.strt = 0
                  prm.xstep = 1
                  prm.dec = setdec%(prm.xstep)
                END IF
              END IF
              prm.xend = prm.strt + prm.axis
            CASE 3
'type 3 axis
              logt% = -1
              thru% = -1
              rvs% = 0
              prm.stp = 1
              IF prm.strt <= 0 THEN prm.cycle = 0'impossible start - try autocalc
              IF prm.cycle <> 0 THEN
                IF prm.strt <= 0 THEN
                  ier% = -1
                  prm.typ = 1
                  prm.stp = 0
                  thru% = 0
                END IF
              ELSE      'cycle=0
'try to fix nonsense inputs            
                IF min >= max OR min <= 0 THEN
                  min = prm.strt
                  max = prm.xend
                END IF
                IF min > 0 AND max > 0 THEN
                  IF min > max THEN
                    rvs% = -1
                    SWAP min, max
                  END IF
                  a% = roundax%(min, max, lb, inc, ub, dec%, -1)
                  IF NOT rvs% THEN
                    prm.strt = lb
                    prm.cycle = prm.axis / inc
                  ELSE
                    prm.strt = ub
                    prm.cycle = -prm.axis / inc
                  END IF
                ELSE
                  ier% = -1
                  prm.typ = 1
                  prm.stp = 0
                  thru% = 0
                END IF
              END IF
              IF NOT ier% THEN
                prm.xend = prm.strt * prm.stp * (10 ^ (prm.axis / prm.cycle))
              END IF
            CASE ELSE
           END SELECT
        LOOP UNTIL (thru%)
        checkax% = ier%
END FUNCTION

FUNCTION drawann% (i%, xp, yp, xs, ys, xe, ye, dx, dy, eps, ang, fstrt, ax AS AxprmT)
'draw the annotation
'input  i%      tick mark number
'       xp,yp   location of tick mark to be annotated in inches
'       xs,ys   location of start of axis in inches
'       xe,ye   location of end of axis in inches
'       dx,dy   offset to lower left corner of annotation
'       eps     tolerance for insuring inclusion of end point annotation
'       ang     angle of annotation
'       fstrt   starting annotation for axis - used for log axes
'       ax      axis specs in type AxprmT
'-------------------------------------------------------------------------
        iret% = 0
        IF xp >= xs AND yp >= ys THEN
          IF xp <= xe + eps AND yp <= ye + eps THEN
            skp% = 0
            IF ax.noend OR ax.nofrst OR ax.nolast THEN
              IF ax.noend THEN
                ax.nofrst = -1
                ax.nolast = -1
              END IF
              SELECT CASE UCASE$(ax.which)
                CASE "X"
                  IF xp < xs + ax.ht AND ax.nofrst THEN skp% = -1
                  IF xp > xe - ax.ht AND ax.nolast THEN skp% = -1
                CASE "Y"
                  IF yp < ys + ax.ht AND ax.nofrst THEN skp% = -1
                  IF yp > ye - ax.ht AND ax.nolast THEN skp% = -1
                CASE ELSE
              END SELECT
            END IF
            IF NOT skp% THEN
              ht = 1.5 * ax.ht
              x = xp + dx
              y = yp + dy
              IF ax.typ = 3 THEN
                fpn = 10
                ndec% = -1
              ELSE
                fpn = ax.strt + i% * ax.stp
                ndec% = ax.dec
              END IF
              CALL Number(x, y, ht, fpn, ang, ndec%)
              IF ax.typ = 3 THEN
                x = xp + dx
                y = yp + dy
                fpn = LOG(fstrt) / LOG(10) + i% * ax.stp
                ndec% = -1
                SELECT CASE UCASE$(ax.which)
                  CASE "X"
                    x = x + 2 * ht
                    y = y + .75 * ht
                    ht = ht / 2
                  CASE "Y"
                    y = y + 2 * ht
                    x = x - .75 * ht
                    ht = ht / 2
                  CASE ELSE
                END SELECT
                CALL Number(x, y, ht, fpn, ang, ndec%)
              END IF
            END IF
          ELSE
            iret% = -1
          END IF
        END IF
        drawann% = iret%
END FUNCTION

FUNCTION drawtick% (xp, yp, xs, ys, xe, ye, eps, dx, dy)
'draws a tick mark on axis
'       xp,yp   location of tick mark to be annotated in inches
'       xs,ys   location of start of axis in inches
'       xe,ye   location of end of axis in inches
'       dx,dy   offset for tick mark
'       eps     tolerance for insuring inclusion of end point annotation
'-------------------------------------------------------------------------
        IF xp >= xs AND yp >= ys THEN
          IF xp <= xe + eps AND yp <= ye + eps THEN
            CALL Plot(xp, yp, 3)
            CALL Plot(xp + dx, yp + dy, 2)
            drawtick% = 0
          ELSE
            drawtick% = -1
          END IF
        END IF
END FUNCTION

SUB eqplt (ind$)
'sets equal scaling of axes
        IF UCASE$(ind$) = "ON" THEN
          axpg.eqplt = -1
        ELSEIF UCASE$(ind$) = "OFF" THEN
          axpg.eqplt = 0
        END IF
END SUB

SUB frame
'frames axis area
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF xprm.set AND yprm.set THEN
          CALL Plot(xprm.org, yprm.org, 3)
          CALL Plot(xprm.org + xprm.axis, yprm.org, 2)
          CALL Plot(xprm.org + xprm.axis, yprm.org + yprm.axis, 2)
          CALL Plot(xprm.org, yprm.org + yprm.axis, 2)
          CALL Plot(xprm.org, yprm.org, 2)
        END IF
END SUB

SUB grid (xgrd%, ygrd%)
'draws grid lines

SHARED xprm AS AxprmT, yprm AS AxprmT
d4% = 4
DIM dx(d4%), dy(d4%)
'vertical grid lines
        IF xgrd% >= 1 AND xprm.set THEN
          xprm.grd = -1
          tckhld% = xprm.stck
          xprm.stck = xgrd%
          dx(1) = xprm.stp * xprm.scl
          dy(1) = 0
          dx(2) = 0
          dy(2) = yprm.axis
          xs = xprm.xpos
          ys = yprm.ypos
          xe = xprm.xpos + xprm.axis - .001
          ye = yprm.ypos + yprm.axis - .001
          CALL tickloop(xprm, dx(), dy(), 0, xs, ys, xe, ye, 1)
          xprm.grd = 0
          xprm.stck = tckhld%
        END IF
'horizontal grid lines
        IF ygrd% >= 1 AND yprm.set THEN
          yprm.grd = -1
          tckhld% = yprm.stck
          yprm.stck = ygrd%
          dx(1) = 0
          dy(1) = yprm.stp * yprm.scl
          dx(2) = xprm.axis
          dy(2) = 0
          xs = xprm.xpos
          ys = yprm.ypos
          xe = xprm.xpos + xprm.axis - .001
          ye = yprm.ypos + yprm.axis - .001
          CALL tickloop(yprm, dx(), dy(), 0, xs, ys, xe, ye, 1)
          yprm.grd = 0
          yprm.stck = tckhld%
        END IF
ERASE dx, dy
END SUB

FUNCTION maxdec% (a, b, c)
'sets default number of decimals to be included in annotation
        a% = setdec%(a)
        b% = setdec%(b)
        d% = a%
        IF b% > d% THEN d% = b%
        IF c <> 1E+20 THEN
          c% = setdec%(c)
          IF c% > d% THEN d% = c%
        END IF
        maxdec% = d%
END FUNCTION

SUB nodraw (x$, y$)
'specifies no drawing of any part of axis
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF UCASE$(x$) = "ON" THEN
          xprm.nodraw = -1
        ELSEIF UCASE$(x$) = "OFF" THEN
          xprm.nodraw = 0
        END IF
        IF UCASE$(y$) = "ON" THEN
          yprm.nodraw = -1
        ELSEIF UCASE$(y$) = "OFF" THEN
          yprm.nodraw = 0
        END IF

END SUB

SUB noend (x$, y$)
'stops plotting of first and last tick and annotation on axis
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF UCASE$(x$) = "ON" THEN
          xprm.noend = -1
        ELSEIF UCASE$(x$) = "OFF" THEN
          xprm.noend = 0
        END IF
        IF UCASE$(y$) = "ON" THEN
          yprm.noend = -1
        ELSEIF UCASE$(y$) = "OFF" THEN
          yprm.noend = 0
        END IF

END SUB

SUB nofrst (x$, y$)
'stops plotting of first tick mark on axis
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF UCASE$(x$) = "ON" THEN
          xprm.nofrst = -1
        ELSEIF UCASE$(x$) = "OFF" THEN
          xprm.nofrst = 0
        END IF
        IF UCASE$(y$) = "ON" THEN
          yprm.nofrst = -1
        ELSEIF UCASE$(y$) = "OFF" THEN
          yprm.nofrst = 0
        END IF
END SUB

SUB nolab (x$, y$)
'specifies that no axis label is to be plotted
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF UCASE$(x$) = "ON" THEN
          xprm.nolab = -1
        ELSEIF UCASE$(x$) = "OFF" THEN
          xprm.nolab = 0
        END IF
        IF UCASE$(y$) = "ON" THEN
          yprm.nolab = -1
        ELSEIF UCASE$(y$) = "OFF" THEN
          yprm.nolab = 0
        END IF

END SUB

SUB nolast (x$, y$)
'eliminate plotting of annotation for last tick
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF UCASE$(x$) = "ON" THEN
          xprm.nolast = -1
        ELSEIF UCASE$(x$) = "OFF" THEN
          xprm.nolast = 0
        END IF
        IF UCASE$(y$) = "ON" THEN
          yprm.nolast = -1
        ELSEIF UCASE$(y$) = "OFF" THEN
          yprm.nolast = 0
        END IF

END SUB

SUB nonum (x$, y$)
'specifies that ticks but no annotation are to be drawn
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF UCASE$(x$) = "ON" THEN
          xprm.nonum = -1
        ELSEIF UCASE$(x$) = "OFF" THEN
          xprm.nonum = 0
        END IF
        IF UCASE$(y$) = "ON" THEN
          yprm.nonum = -1
        ELSEIF UCASE$(y$) = "OFF" THEN
          yprm.nonum = 0
        END IF

END SUB

SUB ResetAxes
'resets both axis parms to defaults
SHARED xprm AS AxprmT, yprm AS AxprmT
DIM prm AS AxprmT
        CALL ResetAxis(prm)
        prm.which = "X"
        xprm = prm
        prm.which = "Y"
        yprm = prm
        axpg.title = ""
        axpg.frame = 0
        axpg.eqplt = 0
END SUB

SUB ResetAxis (prm AS AxprmT)
'resets all axis definition parameters to default
        prm.set = 0
        prm.org = 0
        prm.scl = 1
        prm.xpos = -1
        prm.ypos = -1
        prm.axis = -1
        prm.typ = 1
        prm.strt = 0
        prm.xend = 0
        prm.stp = 0
        prm.xstep = 0
        prm.cycle = 0
        prm.dec = 0
        prm.ht = 0
        prm.side = 0
        prm.stck = 0
        prm.grd = 0
        prm.nodraw = 0
        prm.nonum = 0
        prm.nolab = 0
        prm.nofrst = 0
        prm.nolast = 0
        prm.noend = 0
        prm.nolab = 0
        prm.lab = ""
END SUB

SUB RetAxPrms (prmx AS AxprmT, prmy AS AxprmT)
'returns present axis parms to application prog in AxprmT variables
SHARED xprm AS AxprmT, yprm AS AxprmT
        prmx = xprm
        prmy = yprm
END SUB

FUNCTION roundax% (min, max, lb, inc, ub, dec%, logt%)
'subroutine roundaxis(qqmin,qqmax,qqlb,qqinc,qqub,qqdec%)
' to return suitable axis limits for graphs
'parms  min,max         min and max of variable values
'       lb,inc,ub       returned axis limits and tick increment
'       decs%           decimal accuracy to use
'       logt%           indicates log typ axis (type 3)
'-------------------------------------------------------------------------
        roundax% = 0
        IF min >= max THEN
          roundax% = -1
        ELSE
          IF NOT logt% THEN
'linear axis case - makes sense for graf type only
            lg = LOG((max - min) / 5) / LOG(10)
            dec% = INT(lg)
            SELECT CASE lg - dec%
              CASE IS < .2
                ML = 1
              CASE IS < .5
                ML = 2
              CASE ELSE
                ML = 5
            END SELECT
            inc = ML * 10 ^ dec%
            lb = INT((min - (max - min) / 20) / inc) * inc
            ub = (INT((max + (max - min) / 20) / inc) + 1) * inc
            IF dec% >= 0 THEN dec% = 1
            dec% = -dec%
          ELSE
'log axis case
            IF min <= 0 THEN
              roundax% = -1
            ELSE
              lb = INT(LOG(min) / LOG(10))
              lg = LOG(max) / LOG(10)
              ub = INT(lg)
              IF ub <> lg THEN ub = ub + 1
              inc = ub - lb
              dec% = -1
              lb = 10 ^ lb
              ub = 10 ^ ub
            END IF
          END IF
        END IF
END FUNCTION

FUNCTION sclcoord (prm AS AxprmT, x)
'automatic scaling of a value according to the axis specified
        IF prm.set THEN
          SELECT CASE prm.typ
            CASE 1, 2
              IF prm.scl >= 0 THEN
                org = prm.org
              ELSE
                org = prm.org + prm.axis
              END IF
              sclcoord = org + ABS(prm.scl) * (x - prm.strt)
            CASE 3
              IF x > 0 THEN
                fstrt = fnmin(prm.strt, prm.xend)
                tmp = (LOG(x) - LOG(fstrt)) / LOG(10)
                sclcoord = prm.org + ABS(prm.scl) * tmp
              ELSE
                sclcoord = 0
              END IF
            CASE ELSE
          END SELECT
        ELSE
          sclcoord = x
        END IF
END FUNCTION

SUB scsymbol (x, y, ht, tkiq%, angle, icode%)
'scaled plotting of centered symbol
'parms  x,y     location of symbol in scaled units
'        others are the same as the csymbol plot system parms

SHARED xprm AS AxprmT, yprm AS AxprmT
        CALL Csymbol(sclcoord(xprm, x), sclcoord(yprm, y), ht, tkiq%, angle, icode%)
END SUB

SUB SetAx1 (typ, orig, xstep, xend, prm AS AxprmT)
'sets axis type, origin, step, and end into axis parm variable
'calculates decimals necessary for annotation
        prm.typ = typ
        prm.strt = orig
        prm.xend = xend
        SELECT CASE typ
          CASE 1
            prm.stp = xstep
            prm.dec = maxdec%(prm.strt, prm.stp, prm.xend)
          CASE 2
            prm.xstep = xstep
            prm.dec = maxdec%(prm.strt, prm.xstep, 1E+20)
          CASE 3
            prm.cycle = xstep
          CASE ELSE
        END SELECT
END SUB

SUB SetAxes (xtyp, xorig, xstep, xend, ytyp, yorig, ystep, yend)
'sets axis limits for both axes using only type,orig,step, and end

SHARED xprm AS AxprmT, yprm AS AxprmT
        xprm.which = "X"
        CALL SetAx1(xtyp, xorig, xstep, xend, xprm)
        yprm.which = "Y"
        CALL SetAx1(ytyp, yorig, ystep, yend, yprm)
        a% = axis2%(axpg.xpage, axpg.ypage, xmin, xmax, ymin, ymax, 0, errmsg$)
END SUB

SUB SetAxht2 (xht, yht)
SHARED xprm AS AxprmT, yprm AS AxprmT
        IF xht >= 0 THEN xprm.ht = xht
        IF yht >= 0 THEN yprm.ht = yht
END SUB

SUB SetAxis (wch$, atyp, aorig, astep, aend, alen, alab$, aside%, xpos, ypos)
'sets axis parms for one axis, allows input of all axis parms
SHARED xprm AS AxprmT, yprm AS AxprmT
DIM prm AS AxprmT
        IF UCASE$(wch$) = "X" THEN
          prm = xprm
          opos = ypos
        ELSE
          prm = yprm
          opos = xpos
        END IF
        prm.typ = atyp
        prm.xpos = xpos
        prm.ypos = ypos
        prm.strt = aorig
        prm.xend = aend
        prm.axis = alen
        prm.lab = alab$
        prm.side = aside%
        SELECT CASE atyp
          CASE 1
            prm.stp = astep
            prm.dec = maxdec%(prm.strt, prm.stp, prm.xend)
          CASE 2
            prm.xstep = astep
            prm.dec = maxdec%(prm.strt, prm.xstep, 1E+20)
          CASE 3
            prm.cycle = astep
          CASE ELSE
        END SELECT
        a% = axis1%(prm, opos)
        IF UCASE$(wch$) = "X" THEN
          xprm = prm
        ELSE
          yprm = prm
        END IF
END SUB

SUB SetAxPg (xpage, ypage, tit$, xlab$, ylab$, xlen, ylen)
'sets page variables - labels and titles
SHARED xprm AS AxprmT, yprm AS AxprmT
        axpg.xpage = xpage
        axpg.ypage = ypage
        axpg.title = tit$
        axpg.ht = 0
        xprm.which = "X"
        xprm.lab = xlab$
        xprm.axis = xlen
        yprm.which = "Y"
        yprm.lab = ylab$
        yprm.axis = ylen
END SUB

SUB SetAxPrms (prmx AS AxprmT, prmy AS AxprmT)
'allows transfer of an AxprmT type variable directly from application program
SHARED xprm AS AxprmT, yprm AS AxprmT
        xprm = prmx
        yprm = prmy
END SUB

FUNCTION setdec% (a)
'sets the number of decimals necessary for annotation
        IF a - INT(a) = 0 THEN
          setdec% = -1
        ELSE
          a$ = STR$(a)
          a% = INSTR(a$, ".")
          IF a% = 0 OR a% = LEN(a$) THEN
            setdec% = -1
          ELSE
            setdec% = LEN(a$) - a%
          END IF
        END IF
END FUNCTION

SUB settick (ax AS AxprmT, opos, dx(), dy(), ang, xs, ys, xe, ye, lab$)
'***** general tick parm setting routine
'parms  opos            location of axis in other variable
'       dx(),dy()       offsets for annotation, ticks, and labels
'       ang             rotation angle from positive x axis
'       xs,ys           start loc of axis
'       xe,ye           end loc of axis
'       lab$            axis label
'-------------------------------------------------------------------------
          IF ax.stck <= 0 THEN ax.stck = 1
          IF UCASE$(ax.which) = "X" THEN
            dx(1) = ax.stp * ax.scl
            dy(1) = 0
            dx(2) = 0
            dy(2) = -ax.ht * ax.side
            dx(3) = -1.5 * ax.ht
            dx(4) = ax.axis / 2 - ax.ht * LEN(lab$)
            IF ax.side = 1 THEN
              dy(3) = -3.5 * ax.ht
              dy(4) = -7 * ax.ht
            ELSE
              dy(3) = 2 * ax.ht
              dy(4) = 5 * ax.ht
            END IF
            ang = 0
            lab$ = LTRIM$(RTRIM$(ax.lab))
            xs = ax.xpos
            ys = opos
            xe = ax.xpos + ax.axis
            ye = opos
          ELSE
            dx(1) = 0
            dy(1) = ax.stp * ax.scl
            dx(2) = -ax.ht * ax.side
            dy(2) = 0
            dy(3) = 1.5 * -ax.ht
            dy(4) = ax.axis / 2 - ax.ht * LEN(lab$)
            IF ax.side = 1 THEN
              dx(3) = -2 * ax.ht
              dx(4) = -5 * ax.ht
            ELSE
              dx(3) = 3.5 * ax.ht
              dx(4) = 7 * ax.ht
            END IF
            ang = 90
            lab$ = LTRIM$(RTRIM$(ax.lab))
            xs = opos
            ys = ax.ypos
            xe = opos
            ye = ax.ypos + ax.axis
        END IF
END SUB

SUB sNumber (x, y, ht, fpn, angle, ndec%)
'scaled number routine
'parms  x,y     location of lower left corner in scaled coordinates
'       other parms are the same as the unscaled number routine
'-------------------------------------------------------------------------
SHARED xprm AS AxprmT, yprm AS AxprmT
        CALL Number(sclcoord(xprm, x), sclcoord(yprm, y), ht, fpn, angle, ndec%)
END SUB

SUB splot (x, y, p%)
'scaled pen move routine
'parms  x,y     new pen position in scaled coordinates
'       p%      same as in unscaled version
'-------------------------------------------------------------------------
SHARED xprm AS AxprmT, yprm AS AxprmT
        CALL Plot(sclcoord(xprm, x), sclcoord(yprm, y), p%)
END SUB

SUB spolyline (xary(), yary(), npts%, inc%, lintyp%, tkiq%, ht)
'scaled line plotting routine
'parms  xary(),yary()           'scaled points defining the line
'       other parms are the same as in the unscaled version
'-------------------------------------------------------------------------
SHARED xprm AS AxprmT, yprm AS AxprmT
        DIM x(npts%), y(npts%)
        FOR i% = 0 TO npts%
          x(i%) = sclcoord(xprm, xary(i%))
          y(i%) = sclcoord(yprm, yary(i%))
        NEXT i%
        CALL PolyLine(x(), y(), npts%, inc%, lintyp%, tkiq%, ht)
ERASE x, y
END SUB

SUB ssymbol (x, y, ht, txt$, angle)
'scaled symbol routine
'parms  x,y             scaled location of lower left corner of text
'       other parms are the same as in the unscaled version
'-------------------------------------------------------------------------
SHARED xprm AS AxprmT, yprm AS AxprmT
        CALL Symbol(sclcoord(xprm, x), sclcoord(yprm, y), ht, txt$, angle)
END SUB

SUB tickloop (ax AS AxprmT, dx(), dy(), ang, xs, ys, xe, ye, smul)
'main tick loop
'draws axis, ticks, and annotation
'parms  dx(),dy()       offsets for ticks, annotation, and increments
'       ang             rotation from positive x axis direction
'       xs,ys,xe,ye     start and end locations of axis in inches
'       smul            tick length multiplier - used for subticks
'-------------------------------------------------------------------------
        eps = .001
'draw the axis line
        IF NOT ax.nodraw THEN
          IF NOT ax.grd THEN
            CALL Plot(xs, ys, 3)
            CALL Plot(xe, ye, 2)
          END IF
'main tick loop
          i% = 0
          ret% = 0
          WHILE NOT ret%
            xp = xs + i% * dx(1)
            yp = ys + i% * dy(1)
            IF ax.typ = 3 THEN
              logt% = -1
              min = ax.strt
              max = min * (10 ^ SGN(ax.scl))
              rvs% = 1
              IF min > max THEN
                SWAP min, max
                rvs% = -1
              END IF
              a% = roundax%(min, max, lb, inc, ub, dec%, logt%)
              IF rvs% = -1 THEN lb = ub
              xo = xs + dx(1) * (i% + rvs% * (LOG(lb) - LOG(ax.strt)) / LOG(10))
              yo = ys + dy(1) * (i% + rvs% * (LOG(lb) - LOG(ax.strt)) / LOG(10))
              xp = xo
              yp = yo
              fstrt = lb
            END IF
            IF NOT ax.grd OR i% <> 0 THEN
              ret% = drawtick%(xp, yp, xs, ys, xe, ye, eps, dx(2), dy(2))
            END IF
            IF NOT ax.grd AND NOT ax.nonum THEN
            ret% = drawann%(i%, xp, yp, xs, ys, xe, ye, dx(3), dy(3), eps, ang, fstrt, ax)
            END IF
            IF NOT ret% THEN
              IF NOT logt% THEN
'linear subticks
                k% = 1
                sret% = 0
                IF ax.stck > 1 THEN
                  WHILE k% < ax.stck AND NOT sret%
                    xp = xs + (i% + k% / ax.stck) * dx(1)
                    yp = ys + (i% + k% / ax.stck) * dy(1)
                    sret% = drawtick%(xp, yp, xs, ys, xe, ye, eps, dx(2) * smul, dy(2) * smul)
                    k% = k% + 1
                  WEND
                END IF
              ELSE
'log subticks-j for regular intermediate steps-k for small between these
                      
                j% = 1
                WHILE j% <= 9 AND NOT sret%
                  IF ax.stck > 1 THEN
                    k% = 1
                    lg = (LOG(j% + 1) - LOG(j%)) / (LOG(10) * ax.stck)
                    IF ax.cycle < 0 THEN
                      lg = (LOG(11 - j%) - LOG(11 - j% - 1)) / (LOG(10) * ax.stck)
                    END IF
                    IF lg * (dx(1) + dy(1)) >= tcktol THEN
                      xn = xo
                      yn = yo
                      WHILE k% < ax.stck AND NOT sret%
                        lg = LOG(j% + k% / ax.stck) / LOG(10)
                        IF ax.cycle < 0 THEN
                          lg = 1 - LOG(11 - j% - (ax.stck - k%) / ax.stck) / LOG(10)
                        END IF
                        xp = xo + dx(1) * lg
                        yp = yo + dy(1) * lg
                        sret% = drawtick%(xp, yp, xs, ys, xe, ye, eps, dx(2) * smul, dy(2) * smul)
                        k% = k% + 1
                      WEND
                    END IF
                  END IF
                  j% = j% + 1
                  lg = LOG(j%) / LOG(10)
                  IF ax.cycle < 0 THEN
                    lg = 1 - LOG(11 - j%) / LOG(10)
                  END IF
                  xp = xo + lg * dx(1)
                  yp = yo + lg * dy(1)
                  sret% = drawtick%(xp, yp, xs, ys, xe, ye, eps, dx(2), dy(2))
                WEND
              END IF
              i% = i% + 1
            END IF
          WEND
        END IF
END SUB

SUB xticks (n%)
'sets number of subticks on x axis
SHARED xprm AS AxprmT
        IF n% >= 0 THEN
          xprm.stck = n%
        ELSE
          xprm.stck = 0
        END IF
END SUB

SUB yticks (n%)
'sets number of subticks on y axis
SHARED yprm AS AxprmT
        IF n% >= 0 THEN
          yprm.stck = n%
        ELSE
          yprm.stck = 0
        END IF
END SUB

