'************************* QBHPF.BAS *************************
'  U. S. Geological Survey Computer Program Module QBHPF.BAS
' definitions and graphics routines for HP 7550A file save plotters
' $INCLUDE: 'disclaim.inc'
'*************************************************************************
' $INCLUDE: 'jkplttyp.inc'
' $INCLUDE: 'jk2dcom.inc'

DIM SHARED penmap%(8)            'pen definitions
DIM SHARED tkp$                         'plot string
DIM SHARED xh, yh, ph$, tol             'pen loc and status
DIM SHARED errocc%                      'for error handling

'functions to be called externally
DECLARE FUNCTION HPFWsOpen% (devno%)
DECLARE FUNCTION HPFWsClose% ()
DECLARE FUNCTION HPFDevOpen% (devno%)
DECLARE FUNCTION HPFDevclr% ()
DECLARE SUB HPFDevCmt (row%, c%, llen%, txt$)
DECLARE FUNCTION HPFDevEnd% ()
DECLARE FUNCTION HPFDevMov% (xo, yo, xn, yn, p%)
DECLARE SUB HPFDevPen (spen%)
DECLARE SUB SetHPFnam (nam$)

'internal functions
DECLARE SUB HPFPenUp ()
DECLARE SUB HPFPenDown ()
DECLARE FUNCTION HPFSendString% (p%)
DECLARE SUB Newpen (spen%)

'variables to hold output file no and name
DIM SHARED hpfno%, hpfnam$

'only subs are in this module
END

'error handler
SetErr:
  errocc% = -1
  RESUME NEXT

FUNCTION HPFDevclr%
        errocc% = 0
        ON ERROR GOTO SetErr
        IF hpfnam$ <> "" THEN
'initializes a plot
          IF hpfno% <> 0 THEN
            CLOSE hpfno%
          END IF
          hpfno% = FREEFILE
          OPEN "o", #hpfno%, hpfnam$
          IF NOT errocc% THEN
            WIDTH #hpfno%, 255
            PRINT #hpfno%, "IN;"
          END IF
        ELSE
          errocc% = -1
        END IF
        ON ERROR GOTO 0
        pserr% = errocc%
        HPFDevclr% = errocc%
END FUNCTION

SUB HPFDevCmt (row%, col%, llen%, txt$)
        LOCATE row%, col%
        PRINT SPACE$(llen%);
        LOCATE row%, col%
        PRINT txt$;
END SUB

FUNCTION HPFDevEnd%
'screen/plot termination routine
'endplot return - sends last plot string
       
        CALL Newpen(0)
        IF RIGHT$(tkp$, 1) <> ";" THEN tkp$ = tkp$ + ";"
        tkp$ = tkp$ + "OE;"
        tmperr% = HPFSendString%(999)
        CLOSE #hpfno%
        hpfno% = 0
        HPFDevEnd% = tmperr%
END FUNCTION

FUNCTION HPFDevMov% (xo, yo, xn, yn, p%)
'routine to format instructions for device and send to device
'make local new position vars
        xnl = xn
        ynl = yn
        IF ABS(p% = 2) OR ABS(p% = 999) THEN
'draw case
          IF ABS(p%) = 2 THEN
'move pen to start pos if necessary
            IF ABS(xo - xh) > tol OR ABS(yo - yh) > tol THEN
              IF ph$ <> "PU" THEN CALL HPFPenUp ELSE tkp$ = tkp$ + ","
              tkp$ = tkp$ + STR$(CINT(xo)) + "," + STR$(CINT(yo))
            END IF
'draw to new position
            IF ph$ <> "PD" THEN CALL HPFPenDown ELSE tkp$ = tkp$ + ","
            tkp$ = tkp$ + STR$(CINT(xn)) + "," + STR$(CINT(yn))
'end of plot case
          ELSE
            IF ph$ <> "PU" THEN CALL HPFPenUp ELSE tkp$ = tkp$ + ","
            tkp$ = tkp$ + STR$(CINT(xn)) + "," + STR$(CINT(yn))
'set hold position to impossible - to force pen up to start of new plot
            xnl = -1
            ynl = -1
          END IF
        xh = xnl
        yh = ynl
        END IF
'send string and hold last pen position in any case
        HPFDevMov% = HPFSendString%(p%)
END FUNCTION

FUNCTION HPFDevOpen% (devno%)
        tmperr% = 0
'setting prms for HP 7550A file save
'initialize plot string and pen status
        tkp$ = ""
        ph$ = ""
        st.spen = 0
        tmperr% = 0
        IF st.pws <> devno% THEN tmperr% = HPFWsOpen(devno%)
        HPFDevOpen% = tmperr%
END FUNCTION

SUB HPFDevPen (spen%)
'subroutine for newpen
're-assigns pen if bad selection
'to avoid changing spen%
        IF spen% >= 0 THEN tpen% = spen% ELSE tpen% = 1
        tpen% = penmap%(((tpen% - 1) MOD dev.npen%) + 1)
        IF tpen% <> st.spen THEN
          tkp$ = tkp$ + ";PU;SP" + RIGHT$(STR$(tpen%), 1)
          st.spen = tpen%
          ph$ = ""
        END IF
END SUB

SUB HPFPenDown
'subroutine to start PD sequence
        tkp$ = tkp$ + ";PD"
        ph$ = "PD"
END SUB

SUB HPFPenUp
'subroutine to start PU sequence
        tkp$ = tkp$ + ";PU"
        ph$ = "PU"
END SUB

FUNCTION HPFSendString% (p%)
'to send plot instruction string to plotter
        errocc% = 0
        ON ERROR GOTO SetErr
        PRINT #hpfno%, tkp$
        tkp$ = ""
        ON ERROR GOTO 0
        pserr% = errocc%
        HPFSendString% = errocc%
END FUNCTION

FUNCTION HPFWsClose%
        IF hpfno% <> 0 THEN CLOSE hpfno%
        hpfno% = 0
        st.pws = 0
        PCWsClose% = 0

END FUNCTION

FUNCTION HPFWsOpen% (devno%)
'to specify output device is an HPGL plot file
'set permanent specs here only - file is opened with a call to devclr
        dev.id = "HPGLFILE"
        dev.gmode = 1
        dev.scr.l = 0                'small paper size
        dev.scr.b = 571
        dev.scr.r = 10170
        dev.scr.t = 7840
        dev.upi.x = 1016              'units per inch
        dev.upi.y = 1016
        dev.asp = 1                   'aspect ratio
        dev.yinv = 0                  'not inverted y coordinate
        dev.inch = -1                 'inch-type plotter
        dev.txscr = 0                 'for leaving graphics mode
        dev.npen = 8                  'no of pens-wrong for Sprint,
                                      'but should be ok for most others
        tol = .0001                   'tolerance for same pen position
'pen color mapping variable
        FOR i% = 0 TO 8
          penmap%(i%) = i%
        NEXT i%
       
        st.pws = devno%
        pserr% = errocc%
        HPFWsOpen% = errocc%
END FUNCTION

SUB SetHPFnam (nam$)
        hpfnam$ = nam$
END SUB

